@@ -56,7 +56,13 @@ static int mi_interp_query_hook (const char *ctlstr, va_list ap)
 
 static void mi_insert_notify_hooks (void);
 static void mi_remove_notify_hooks (void);
+
+static void mi_on_signal_received (enum gdb_signal siggnal);
+static void mi_on_end_stepping_range (void);
+static void mi_on_signal_exited (enum gdb_signal siggnal);
+static void mi_on_exited (int exitstatus);
 static void mi_on_normal_stop (struct bpstats *bs, int print_frame);
+static void mi_on_no_history (void);
 
 static void mi_new_thread (struct thread_info *t);
 static void mi_thread_exit (struct thread_info *t, int silent);
@@ -79,6 +85,7 @@ static void mi_breakpoint_modified (struct breakpoint *b);
 static void mi_command_param_changed (const char *param, const char *value);
 static void mi_memory_changed (struct inferior *inf, CORE_ADDR memaddr,
 			       ssize_t len, const bfd_byte *myaddr);
+static void mi_on_sync_execution_done (void);
 
 static int report_initial_inferior (struct inferior *inf, void *closure);
 
@@ -118,7 +125,16 @@ mi_interpreter_init (struct interp *interp, int top_level)
   else
     gdb_assert_not_reached ("unhandled MI version");
 
-  mi->uiout = mi_out_new (mi_version);
+  mi->mi_uiout = mi_out_new (mi_version);
+  mi->cli_uiout = cli_out_new (mi->out);
+
+  /* There are installed even if MI is not the top level interpreter.
+     The callbacks themselves decide whether to be skipped.  */
+  observer_attach_signal_received (mi_on_signal_received);
+  observer_attach_end_stepping_range (mi_on_end_stepping_range);
+  observer_attach_signal_exited (mi_on_signal_exited);
+  observer_attach_exited (mi_on_exited);
+  observer_attach_no_history (mi_on_no_history);
 
   if (top_level)
     {
@@ -143,6 +159,7 @@ mi_interpreter_init (struct interp *interp, int top_level)
       observer_attach_breakpoint_modified (mi_breakpoint_modified);
       observer_attach_command_param_changed (mi_command_param_changed);
       observer_attach_memory_changed (mi_memory_changed);
+      observer_attach_sync_execution_done (mi_on_sync_execution_done);
 
       /* The initial inferior is created before this function is
 	 called, so we need to report it explicitly.  Use iteration in
@@ -208,14 +225,6 @@ mi_interpreter_exec (void *data, const char *command)
   return exception_none;
 }
 
-/* Never display the default GDB prompt in MI case.  */
-
-static int
-mi_interpreter_prompt_p (void *data)
-{
-  return 0;
-}
-
 void
 mi_cmd_interpreter_exec (char *command, char **argv, int argc)
 {
@@ -297,6 +306,28 @@ mi_execute_command_wrapper (const char *cmd)
   mi_execute_command (cmd, stdin == instream);
 }
 
+/* Observer for the synchronous_command_done notification.  */
+
+static void
+mi_on_sync_execution_done (void)
+{
+  /* MI generally prints a prompt after a command, indicating it's
+     ready for further input.  However, due to an historical wart, if
+     MI async, and a (CLI) synchronous command was issued, then we
+     will print the prompt right after printing "^running", even if we
+     cannot actually accept any input until the target stops.  See
+     mi_on_resume.  However, if the target is async but MI is sync,
+     then we need to output the MI prompt now, to replicate gdb's
+     behavior when neither the target nor MI are async.  (Note this
+     observer is only called by the asynchronous target event handling
+     code.)  */
+  if (!mi_async_p ())
+    {
+      fputs_unfiltered ("(gdb) \n", raw_stdout);
+      gdb_flush (raw_stdout);
+    }
+}
+
 /* mi_execute_command_wrapper wrapper suitable for INPUT_HANDLER.  */
 
 static void
@@ -304,8 +335,24 @@ mi_execute_command_input_handler (char *cmd)
 {
   mi_execute_command_wrapper (cmd);
 
-  fputs_unfiltered ("(gdb) \n", raw_stdout);
-  gdb_flush (raw_stdout);
+  /* MI generally prints a prompt after a command, indicating it's
+     ready for further input.  However, due to an historical wart, if
+     MI is async, and a synchronous command was issued, then we will
+     print the prompt right after printing "^running", even if we
+     cannot actually accept any input until the target stops.  See
+     mi_on_resume.
+
+     If MI is not async, then we print the prompt when the command
+     finishes.  If the target is sync, that means output the prompt
+     now, as in that case executing a command doesn't return until the
+     command is done.  However, if the target is async, we go back to
+     the event loop and output the prompt in the
+     'synchronous_command_done' observer.  */
+  if (!target_is_async_p () || !sync_execution)
+    {
+      fputs_unfiltered ("(gdb) \n", raw_stdout);
+      gdb_flush (raw_stdout);
+    }
 }
 
 static void
@@ -431,14 +478,111 @@ restore_current_uiout_cleanup (void *arg)
   current_uiout = saved_uiout;
 }
 
-/* Cleanup that destroys the a ui_out object.  */
+/* Return the MI interpreter, if it is active -- either because it's
+   the top-level interpreter or the interpreter executing the current
+   command.  Returns NULL if the MI interpreter is not being used.  */
+
+static struct interp *
+find_mi_interpreter (void)
+{
+  struct interp *interp;
+
+  interp = top_level_interpreter ();
+  if (ui_out_is_mi_like_p (interp_ui_out (interp)))
+    return interp;
+
+  interp = command_interp ();
+  if (ui_out_is_mi_like_p (interp_ui_out (interp)))
+    return interp;
+
+  return NULL;
+}
+
+/* Return the MI_INTERP structure of the active MI interpreter.
+   Returns NULL if MI is not active.  */
+
+static struct mi_interp *
+mi_interp_data (void)
+{
+  struct interp *interp = find_mi_interpreter ();
+
+  if (interp != NULL)
+    return interp_data (interp);
+  return NULL;
+}
+
+/* Observers for several run control events that print why the
+   inferior has stopped to both the the MI event channel and to the MI
+   console.  If the MI interpreter is not active, print nothing.  */
+
+/* Observer for the signal_received notification.  */
 
 static void
-ui_out_free_cleanup (void *arg)
+mi_on_signal_received (enum gdb_signal siggnal)
 {
-  struct ui_out *uiout = arg;
+  struct mi_interp *mi = mi_interp_data ();
 
-  ui_out_destroy (uiout);
+  if (mi == NULL)
+    return;
+
+  print_signal_received_reason (mi->mi_uiout, siggnal);
+  print_signal_received_reason (mi->cli_uiout, siggnal);
+}
+
+/* Observer for the end_stepping_range notification.  */
+
+static void
+mi_on_end_stepping_range (void)
+{
+  struct mi_interp *mi = mi_interp_data ();
+
+  if (mi == NULL)
+    return;
+
+  print_end_stepping_range_reason (mi->mi_uiout);
+  print_end_stepping_range_reason (mi->cli_uiout);
+}
+
+/* Observer for the signal_exited notification.  */
+
+static void
+mi_on_signal_exited (enum gdb_signal siggnal)
+{
+  struct mi_interp *mi = mi_interp_data ();
+
+  if (mi == NULL)
+    return;
+
+  print_signal_exited_reason (mi->mi_uiout, siggnal);
+  print_signal_exited_reason (mi->cli_uiout, siggnal);
+}
+
+/* Observer for the exited notification.  */
+
+static void
+mi_on_exited (int exitstatus)
+{
+  struct mi_interp *mi = mi_interp_data ();
+
+  if (mi == NULL)
+    return;
+
+  print_exited_reason (mi->mi_uiout, exitstatus);
+  print_exited_reason (mi->cli_uiout, exitstatus);
+}
+
+/* Observer for the no_history notification.  */
+
+static void
+mi_on_no_history (void)
+{
+  struct mi_interp *mi = mi_interp_data ();
+
+  if (mi == NULL)
+    return;
+
+  print_no_history_reason (mi->mi_uiout);
+  print_no_history_reason (mi->cli_uiout);
 }
 
 static void
@@ -506,16 +650,12 @@ mi_on_normal_stop (struct bpstats *bs, int print_frame)
 	      struct mi_interp *mi = top_level_interpreter_data ();
 	      struct target_waitstatus last;
 	      ptid_t last_ptid;
-	      struct ui_out *cli_uiout;
 	      struct cleanup *old_chain;
 
-	      /* Sets the current uiout to a new temporary CLI uiout
-		 assigned to STREAM.  */
-	      cli_uiout = cli_out_new (mi->out);
-	      old_chain = make_cleanup (ui_out_free_cleanup, cli_uiout);
-
-	      make_cleanup (restore_current_uiout_cleanup, current_uiout);
-	      current_uiout = cli_uiout;
+	      /* Set the current uiout to CLI uiout temporarily.  */
+	      old_chain = make_cleanup (restore_current_uiout_cleanup,
+					current_uiout);
+	      current_uiout = mi->cli_uiout;
 
 	      get_last_target_status (&last_ptid, &last);
 	      print_stop_event (&last);
@@ -828,10 +968,10 @@ mi_on_resume (ptid_t ptid)
       running_result_record_printed = 1;
       /* This is what gdb used to do historically -- printing prompt even if
 	 it cannot actually accept any input.  This will be surely removed
-	 for MI3, and may be removed even earler.  */
-      /* FIXME: review the use of target_is_async_p here -- is that
-	 what we want? */
-      if (!target_is_async_p ())
+	 for MI3, and may be removed even earlier.  SYNC_EXECUTION is
+	 checked here because we only need to emit a prompt if a
+	 synchronous command was issued when the target is async.  */
+      if (!target_is_async_p () || sync_execution)
 	fputs_unfiltered ("(gdb) \n", raw_stdout);
     }
   gdb_flush (raw_stdout);
@@ -972,7 +1112,7 @@ mi_ui_out (struct interp *interp)
 {
   struct mi_interp *mi = interp_data (interp);
 
-  return mi->uiout;
+  return mi->mi_uiout;
 }
 
 /* Save the original value of raw_stdout here when logging, so we can
@@ -1034,7 +1174,6 @@ _initialize_mi_interp (void)
       mi_interpreter_resume,	/* resume_proc */
       mi_interpreter_suspend,	/* suspend_proc */
       mi_interpreter_exec,	/* exec_proc */
-      mi_interpreter_prompt_p,	/* prompt_proc_p */
       mi_ui_out, 		/* ui_out_proc */
       mi_set_logging,		/* set_logging_proc */
       mi_command_loop		/* command_loop_proc */
